__author__ = 'wzhvictor@outlook.com'

cdef extern from *:
    ctypedef char* const_char_ptr "const char*"


cdef extern from "stdlib.h":
    void* malloc(size_t)
    void free(void* )


cdef extern from "libinjection_sqli.h":
    ctypedef struct c_stoken_t "stoken_t":
        size_t  pos
        size_t  len

        int     count

        char    type
        char    str_open
        char    str_close
        char    val[32]

    ctypedef struct c_sfilter "sfilter":
        const_char_ptr  s
        size_t          slen

        int             flags
        size_t          pos

        c_stoken_t      tokenvec[8]
        c_stoken_t      *current
        char            fingerprint[8]

        int             reason
        int             stats_comment_ddw
        int             stats_comment_ddx
        int             stats_comment_c
        int             stats_comment_hash
        int             stats_folds
        int             stats_tokens

    void libinjection_sqli_init(c_sfilter *,
                                const_char_ptr,
                                size_t,
                                int)

    bint libinjection_is_sqli(c_sfilter *)


cdef extern from "libinjection_xss.h":
    bint libinjection_is_xss(const_char_ptr,
                             size_t,
                             int)



__author__ = 'wzhvictor@outlook.com'

def is_sql_injection(payload):
    cdef c_sfilter *sfp = <c_sfilter *> malloc(sizeof(c_sfilter))
    encoded_payload = payload.encode('utf_8')
    libinjection_sqli_init(sfp, encoded_payload, len(encoded_payload), 0)
    res = dict(
        is_sqli=libinjection_is_sqli(sfp),
        fingerprint=sfp.fingerprint.decode('utf_8'),
        token_vector=sfp.tokenvec
    )
    free(sfp)
    return res

def is_xss(payload):
    encoded_payload = payload.encode('utf_8')
    for flag in range(5):
        if libinjection_is_xss(encoded_payload, len(encoded_payload), flag):
            return dict(
                is_xss=True,
                flag=flag
            )
    return dict(
        is_xss=False,
        flag=-1,
    )



from setuptools import setup, Extension
from Cython.Distutils import build_ext

source_files = [
    'libinjection/libinjection.pyx',
    'libinjection/src/libinjection_sqli.c',
    'libinjection/src/libinjection_html5.c',
    'libinjection/src/libinjection_xss.c'
]
cmd_class = {'build_ext': build_ext}

setup(
    name='libinjection-python',
    version='1.1.5',
    author='wzhvictor',
    author_email='wzhvictor@outlook.com',
    url='https://github.com/wzhvictor/libinjection-python',
    description='Libinjection Python Wrapper',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Cython',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    packages=[],
    include_package_data=True,
    cmdclass=cmd_class,
    ext_modules=[Extension(
        name='libinjection',
        sources=source_files,
        include_dirs=['libinjection/src'],
        library_dirs=['libinjection/src'])
    ],
    setup_requires=[
        'setuptools>=38.3.0',
        'Cython>=0.23'
    ]
)
